local addon = LibStub("AceAddon-3.0"):NewAddon("Langoustine","AceEvent-3.0","AceConsole-3.0","LibSink-2.0","AceTimer-3.0", "AceBucket-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("Langoustine",false)

local defaults = {
	profile = {
		TankShow=true,
		TankOptions = {
			x=0, 
			y=0, 
			locked=true, 
			maxTanks = 10, 
			unitHeight=10, 
			unitWidth=10, 
			font="", 
			barTexture="Blizzard", 
			growth="Down", 
			icon=true, 
			alpha=1,  
			showAggro=true,
			colorByClass=true,
			backDrop="",
			scale=1,
			inverseHealth = false,
			enableClickCast = true,
			reverseOrder=false,
		},
		TankShowTarget=true,
		TankShowTargetTarget=true,
		AutoPromote={ ["Nurofen"] = true,},
		AutoLoot={ method="master", threshold="4", looter="Michelina"},
		AutoPromoteEnabled=true,
		AutoLootEnabled=true,
		Minimap = {},
		Sink={},
		NotifyMTDeath=true,
		WhisperInviteTriggger="invite",
		WhisperInvite=true,
		ReadyCheck={},
		SetML = false,
		SetLoot = false,
		SetThreshold = false,
	},
}
local guildRanks = {}
local guildMembers = {}
local wipe,tinsert = wipe, tinsert
local raiders = {}
-- upcasts of function we wil use a lot
local isRaidOfficer = isRaidOfficer
local GetNumPartyMembers = GetNumPartyMembers
local UnitInRaid = UnitInRaid
local GetNumRaidMembers = GetNumRaidMembers
local ConvertToRaid = ConvertToRaid
local IsInInstance = IsInInstance
local GetRaidRosterInfo = GetRaidRosterInfo
-- Timer callbacks
local ltTimer = nil

function addon:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("LangoustineDB", defaults, "Default")
	self:RegisterBucketEvent("RAID_ROSTER_UPDATE",1,"RaidCheck")
	self:RegisterEvent("GUILD_ROSTER_UPDATE","GuildCheck")
	self:RegisterEvent("PLAYER_ENTERING_WORLD","RaidCheck")
	self:RegisterChatCommand("radisband", function() addon:Disband() end)
end

-- when we disable ourselves
function addon:DisableEvents()
	self:RegisterBucketEvent("RAID_ROSTER_UPDATE",1,"RaidCheck")
	self:RegisterEvent("GUILD_ROSTER_UPDATE","GuildCheck")
	self:UnregisterEvent("CHAT_MSG_WHISPER")
	self.db.profile.SetML = false
	self.db.profile.SetLoot = false
	self.db.profile.SetThreshold = false
end

function addon:OnEnable()
	self:EnableEvents()
end
-- enable ourselves
function addon:EnableEvents()
	self:RegisterEvent("CHAT_MSG_WHISPER","CheckInvite")
end


function addon:CheckInvite(event, message, player)
	print("Got a whisper! "..message)
	if (IsRaidOfficer()) then
		if self.db.profile.WhisperInvite and message:lower() == self.db.profile.WhisperInviteTriggger then
			print("Trying to add player!")
			local raiders = GetNumRaidMembers()
			if raiders < 40 then
				InviteUnit(player)
			else
				SendChatMessage("The raid is full at this time.", "WHISPER", nil, player)
			end
		end
	end
end
-- update local rank and member list
function addon:GuildCheck()
	wipe(guildRanks)
	wipe(guildMembers)	
	for i = 1, GuildControlGetNumRanks() do
		tinsert(guildRanks, GuildControlGetRankName(i))
	end
	local numGuildMembers = GetNumGuildMembers()
	for i = 1, numGuildMembers do
		local name = GetGuildRosterInfo(i)
		if name then
			guildMembers[name] = true
		end
	end
end

-- conver the party to raid
function addon:ConvertToRaid()
	if GetNumPartyMembers() > 0 and not UnitInRaid("player") then
		self:UnregisterEvent("PARTY_MEMBERS_CHANGED")
		ConvertToRaid()
	end
end

-- disband the raid
function addon:Disband()
	if (IsRaidOfficer()) then
		local player = UnitName("player")
		SendChatMessage("Raid being disbanded by "..player,"RAID")
		for i = 1, GetNumRaidMembers() do
			local name = GetRaidRosterInfo(i)
			if name ~= player then
				UninviteUnit(name)
			end
		end
		LeaveParty()
		self:DisableEvents()
	end
end

-- invite by guild rank or by level
function addon:Invite(byRank,byLevel)
	self:RegisterEvent("PARTY_MEMBERS_CHANGED","ConvertToRaid")
	wipe(raiders)
	for i = 1, GetNumGuildMembers() do
		local name, _, rank, level, _, _ , _, _, online = GetGuildRosterInfo(i)
		-- check for both party and raid since we may run invite after grabbign a few people
		if name and online and not UnitInParty(name) and not UnitInRaid(name) then
			if byLevel and level <= byLevel then
				tinsert(raiders, name)
			elseif byRank and (rank + 1) <= byRank then
				tinsert(raiders, name)
			end
		end
	end
	-- invite everyone
	for _,name in ipairs(raiders) do
		InviteUnit(name)
	end
	wipe(raiders)
end
function addon:SetLootThreshold()
	local thold = GetLootThreshold()
	if thold ~= tonumber(self.db.profile.AutoLoot.threshold) then
		SetLootThreshold(self.db.profile.AutoLoot.threshold)
	end
	ltTimer = nil
end
-- Raid Checks
function addon:RaidCheck()
	-- Are we in a raid?
	local inRaid = UnitInRaid("player")
	local _, instanceType = IsInInstance()
	if instanceType == "pvp" or instanceType == "arena" then return end
	-- if we are not enabled and joined a raid enable ourselves
	if not self:IsEnabled() and inRaid then
		self:EnableEvents()
	elseif self:IsEnabled() and not inRaid then -- left the raid group disable ourselves
		self:DisableEvents()
		return
	end
	-- Raid leader checks
	if (IsRaidLeader()) then
		-- Check promote list
		if self.db.profile.AutoPromoteEnabled then
			for i = 1, GetNumRaidMembers() do
				local name, rank = GetRaidRosterInfo(i)
				if name and rank == 0 and self.db.profile.AutoPromote[name] then
					PromoteToAssistant(name)
				end
			end
		end
		-- Check Loot setup
		if self.db.profile.AutoLootEnabled then
			local method,_,masterLooter = GetLootMethod()
			if not self.db.profile.SetLoot then
				-- Change Method
				if method ~= self.db.profile.AutoLoot.method and not self.db.profile.SetLoot then
					if self.db.profile.AutoLoot.method == "master" then
						local mlCandidate = UnitName("player")
						for i = 1, GetNumRaidMembers() do
							local name, rank = GetRaidRosterInfo(i)
							if name == self.db.profile.AutoLoot.looter then
								mlCandidate = name
							end
						end
						SetLootMethod(self.db.profile.AutoLoot.method,mlCandidate)
						self.db.profile.SetML = false
					else
						SetLootMethod(self.db.profile.AutoLoot.method,self.db.profile.AutoLoot.threshold)
						self.db.profile.SetML = true
					end
				end
				self.db.profile.SetLoot = true
				local thold = GetLootThreshold()
				if thold == tonumber(self.db.profile.AutoLoot.threshold) then
					if ltTimer then
						self:CancelTimer(ltTimer)
					end
				else
					if ltTimer == nil then
						ltTimer = self:ScheduleTimer("SetLootThreshold",2)
					end
				end
			end			
			if method == "master" and self.db.profile.AutoLoot.looter and not self.db.profile.SetML then
				-- do we have the right master looter?
				local looterName = GetRaidRosterInfo(masterLooter)
				if looterName ~= self.db.profile.AutoLoot.looter then
					-- is the looter we want in the raid?
					for i = 1, GetNumRaidMembers() do
						local name, rank = GetRaidRosterInfo(i)
						if name == self.db.profile.AutoLoot.looter then
							SetLootMethod(method,name,self.db.profile.AutoLoot.threshold)
							self.db.profile.SetML = true
							-- we set the ml according to prefs, stop checking anymore till we leave the raid
							-- during a run the raidleader may reset loot as he wants and/or the master looter
						end
					end
				end
			end
		end
	end
end